@@ -2,6 +2,7 @@ package config
 
 import (
 	"fmt"
+	"io/ioutil"
 	"net"
 	"net/url"
 	"os"
@@ -27,11 +28,12 @@ var (
 
 	// Default holds parsed config file parameters and command-line overrides
 	Default = DefaultConfig{
-		MTU:             1400,
-		ConntrackZone:   64000,
-		EncapType:       "geneve",
-		EncapIP:         "",
-		InactivityProbe: 100000,
+		MTU:               1400,
+		ConntrackZone:     64000,
+		EncapType:         "geneve",
+		EncapIP:           "",
+		InactivityProbe:   100000,
+		RawClusterSubnets: "10.128.0.0/14/23",
 	}
 
 	// Logging holds logging-related parsed config file parameters and command-line overrides
@@ -49,7 +51,9 @@ var (
 
 	// Kubernetes holds Kubernetes-related parsed config file parameters and command-line overrides
 	Kubernetes = KubernetesConfig{
-		APIServer: "http://localhost:8080",
+		APIServer:          "http://localhost:8080",
+		ServiceCIDR:        "172.16.1.0/24",
+		OVNConfigNamespace: "ovn-kubernetes",
 	}
 
 	// OvnNorth holds northbound OVN database client and server authentication and location details
@@ -57,6 +61,15 @@ var (
 
 	// OvnSouth holds southbound OVN database client and server authentication and location details
 	OvnSouth OvnAuthConfig
+
+	// Gateway holds node gateway-related parsed config file parameters and command-line overrides
+	Gateway GatewayConfig
+)
+
+const (
+	kubeServiceAccountPath       string = "/var/run/secrets/kubernetes.io/serviceaccount/"
+	kubeServiceAccountFileToken  string = "token"
+	kubeServiceAccountFileCACert string = "ca.crt"
 )
 
 // DefaultConfig holds parsed config file parameters and command-line overrides
@@ -76,6 +89,12 @@ type DefaultConfig struct {
 	// Maximum number of milliseconds of idle time on connection that
 	// ovn-controller waits before it will send a connection health probe.
 	InactivityProbe int `gcfg:"inactivity-probe"`
+	// RawClusterSubnets holds the unparsed cluster subnets. Should only be
+	// used inside config module.
+	RawClusterSubnets string `gcfg:"cluster-subnets"`
+	// ClusterSubnets holds parsed cluster subnet entries and may be used
+	// outside the config module.
+	ClusterSubnets []CIDRNetworkEntry
 }
 
 // LoggingConfig holds logging-related parsed config file parameters and command-line overrides
@@ -98,10 +117,40 @@ type CNIConfig struct {
 
 // KubernetesConfig holds Kubernetes-related parsed config file parameters and command-line overrides
 type KubernetesConfig struct {
-	Kubeconfig string `gcfg:"kubeconfig"`
-	CACert     string `gcfg:"cacert"`
-	APIServer  string `gcfg:"apiserver"`
-	Token      string `gcfg:"token"`
+	Kubeconfig         string `gcfg:"kubeconfig"`
+	CACert             string `gcfg:"cacert"`
+	APIServer          string `gcfg:"apiserver"`
+	Token              string `gcfg:"token"`
+	ServiceCIDR        string `gcfg:"service-cidr"`
+	OVNConfigNamespace string `gcfg:"ovn-config-namespace"`
+}
+
+// GatewayMode holds the node gateway mode
+type GatewayMode string
+
+const (
+	// GatewayModeDisabled indicates the node gateway mode is disabled
+	GatewayModeDisabled GatewayMode = ""
+	// GatewayModeShared indicates OVN shares a gateway interface with the node
+	GatewayModeShared GatewayMode = "shared"
+	// GatewayModeSpare indicates OVN claims a spare interfaface for the gateway
+	GatewayModeSpare GatewayMode = "spare"
+	// GatewayModeLocal indicates OVN creates a local NAT-ed interface for the gateway
+	GatewayModeLocal GatewayMode = "local"
+)
+
+// GatewayConfig holds node gateway-related parsed config file parameters and command-line overrides
+type GatewayConfig struct {
+	// Mode is the gateway mode; if may be either empty (disabled), "shared", "spare", or "local"
+	Mode GatewayMode `gcfg:"mode"`
+	// Interface is the network interface to use for the gateway in "shared" or "spare" mode
+	Interface string `gcfg:"interface"`
+	// NextHop is the gateway IP address of Interface; will be autodetected if not given
+	NextHop string `gcfg:"next-hop"`
+	// VLANID is the option VLAN tag to apply to gateway traffic for "shared" or "spare" modes
+	VLANID uint `gcfg:"vlan-id"`
+	// NodeportEnable sets whether to provide Kubernetes NodePort service or not
+	NodeportEnable bool `gcfg:"nodeport"`
 }
 
 // OvnAuthConfig holds client authentication and location details for
@@ -140,6 +189,7 @@ type config struct {
 	Kubernetes KubernetesConfig
 	OvnNorth   OvnAuthConfig
 	OvnSouth   OvnAuthConfig
+	Gateway    GatewayConfig
 }
 
 var (
@@ -149,6 +199,18 @@ var (
 	savedKubernetes KubernetesConfig
 	savedOvnNorth   OvnAuthConfig
 	savedOvnSouth   OvnAuthConfig
+	savedGateway    GatewayConfig
+
+	// legacy service-cluster-ip-range CLI option
+	serviceClusterIPRange string
+	// legacy cluster-subnet CLI option
+	clusterSubnet string
+	// legacy init-gateways CLI option
+	initGateways bool
+	// legacy gateway-spare-interface CLI option
+	gatewaySpareInterface bool
+	// legacy gateway-local CLI option
+	gatewayLocal bool
 )
 
 func init() {
@@ -159,7 +221,9 @@ func init() {
 	savedKubernetes = Kubernetes
 	savedOvnNorth = OvnNorth
 	savedOvnSouth = OvnSouth
+	savedGateway = Gateway
 	Flags = append(Flags, CommonFlags...)
+	Flags = append(Flags, CNIFlags...)
 	Flags = append(Flags, K8sFlags...)
 	Flags = append(Flags, OvnNBFlags...)
 	Flags = append(Flags, OvnSBFlags...)
@@ -175,6 +239,7 @@ func RestoreDefaultConfig() {
 	Kubernetes = savedKubernetes
 	OvnNorth = savedOvnNorth
 	OvnSouth = savedOvnSouth
+	Gateway = savedGateway
 }
 
 // copy members of struct 'src' into the corresponding field in struct 'dst'
@@ -223,6 +288,14 @@ func overrideFields(dst, src interface{}) {
 			if srcField.Int() != 0 {
 				dstField.Set(srcField)
 			}
+		case reflect.Uint:
+			if srcField.Uint() != 0 {
+				dstField.Set(srcField)
+			}
+		case reflect.Bool:
+			if srcField.Bool() {
+				dstField.Set(srcField)
+			}
 		default:
 			panic(fmt.Sprintf("unhandled struct %q field %q type %v",
 				dstType.Name(), structField.Name, srcField.Kind()))
@@ -239,10 +312,6 @@ var cliConfig config
 //CommonFlags capture general options.
 var CommonFlags = []cli.Flag{
 	// Mode flags
-	cli.BoolFlag{
-		Name:  "net-controller",
-		Usage: "Flag to start the central controller that watches pods/services/policies",
-	},
 	cli.StringFlag{
 		Name:  "init-master",
 		Usage: "initialize master, requires the hostname as argument",
@@ -251,14 +320,14 @@ var CommonFlags = []cli.Flag{
 		Name:  "init-node",
 		Usage: "initialize node, requires the name that node is registered with in kubernetes cluster",
 	},
+	cli.StringFlag{
+		Name:  "cleanup-node",
+		Usage: "cleanup node, requires the name that node is registered with in kubernetes cluster",
+	},
 	cli.StringFlag{
 		Name:  "pidfile",
 		Usage: "Name of file that will hold the ovnkube pid (optional)",
 	},
-	cli.BoolFlag{
-		Name:  "ha",
-		Usage: "HA option to reconstruct OVN database after failover",
-	},
 	cli.StringFlag{
 		Name:  "config-file",
 		Usage: "configuration file path (default: /etc/openvswitch/ovn_k8s.conf)",
@@ -289,6 +358,24 @@ var CommonFlags = []cli.Flag{
 			"connection for ovn-controller before it sends a inactivity probe",
 		Destination: &cliConfig.Default.InactivityProbe,
 	},
+	cli.StringFlag{
+		Name:        "cluster-subnet",
+		Usage:       "Deprecated alias for cluster-subnets.",
+		Destination: &clusterSubnet,
+	},
+	cli.StringFlag{
+		Name:  "cluster-subnets",
+		Value: "10.128.0.0/14/23",
+		Usage: "A comma separated set of IP subnets and the associated " +
+			"hostsubnet prefix lengths to use for the cluster (eg, \"10.128.0.0/14/23,10.0.0.0/14/23\"). " +
+			"Each entry is given in the form [IP address/prefix-length/hostsubnet-prefix-length] " +
+			"and cannot overlap with other entries. The hostsubnet-prefix-length " +
+			"is optional and if unspecified defaults to 24. The " +
+			"hostsubnet-prefix-length defines how many IP addresses " +
+			"are dedicated to each node and may be different for each " +
+			"entry.",
+		Destination: &cliConfig.Default.RawClusterSubnets,
+	},
 
 	// Logging options
 	cli.IntFlag{
@@ -303,17 +390,8 @@ var CommonFlags = []cli.Flag{
 	},
 }
 
-// K8sFlags capture Kubernetes-related options
-var K8sFlags = []cli.Flag{
-	cli.StringFlag{
-		Name:  "cluster-subnet",
-		Value: "11.11.0.0/16",
-		Usage: "A comma separated set of IP subnets and the associated" +
-			"hostsubnetlengths to use for the cluster (eg, \"10.128.0.0/14/23,10.0.0.0/14/23\"). " +
-			"Each entry is given in the form IP address/subnet mask/hostsubnetlength, " +
-			"the hostsubnetlength is optional and if unspecified defaults to 24. The " +
-			"hostsubnetlength defines how many IP addresses are dedicated to each node.",
-	},
+// CNIFlags capture CNI-related options
+var CNIFlags = []cli.Flag{
 	// CNI options
 	cli.StringFlag{
 		Name:        "cni-conf-dir",
@@ -330,12 +408,22 @@ var K8sFlags = []cli.Flag{
 		Usage:       "the ID of the HNS network to which containers will be attached (default: not set)",
 		Destination: &cliConfig.CNI.WinHNSNetworkID,
 	},
+}
+
+// K8sFlags capture Kubernetes-related options
+var K8sFlags = []cli.Flag{
 	cli.StringFlag{
-		Name: "service-cluster-ip-range",
+		Name:        "service-cluster-ip-range",
+		Usage:       "Deprecated alias for k8s-service-cidr.",
+		Destination: &serviceClusterIPRange,
+	},
+	cli.StringFlag{
+		Name: "k8s-service-cidr",
 		Usage: "A CIDR notation IP range from which k8s assigns " +
 			"service cluster IPs. This should be the same as the one " +
 			"provided for kube-apiserver \"-service-cluster-ip-range\" " +
-			"option.",
+			"option. (default: 172.16.1.0/24)",
+		Destination: &cliConfig.Kubernetes.ServiceCIDR,
 	},
 	cli.StringFlag{
 		Name:        "k8s-kubeconfig",
@@ -357,6 +445,11 @@ var K8sFlags = []cli.Flag{
 		Usage:       "the Kubernetes API authentication token (not required if --k8s-kubeconfig is given)",
 		Destination: &cliConfig.Kubernetes.Token,
 	},
+	cli.StringFlag{
+		Name:        "ovn-config-namespace",
+		Usage:       "specify a namespace which will contain services to config the OVN databases",
+		Destination: &cliConfig.Kubernetes.OVNConfigNamespace,
+	},
 }
 
 // OvnNBFlags capture OVN northbound database options
@@ -413,9 +506,10 @@ var OvnSBFlags = []cli.Flag{
 
 //OVNGatewayFlags capture L3 Gateway related flags
 var OVNGatewayFlags = []cli.Flag{
-	cli.BoolFlag{
-		Name:  "init-gateways",
-		Usage: "initialize a gateway in the minion. Only useful with \"init-node\"",
+	cli.StringFlag{
+		Name: "gateway-mode",
+		Usage: "Sets the cluster gateway mode. One of \"shared\", \"spare\", " +
+			"or \"local\". If not given gateway functionality is disabled.",
 	},
 	cli.StringFlag{
 		Name: "gateway-interface",
@@ -423,6 +517,7 @@ var OVNGatewayFlags = []cli.Flag{
 			"If none specified, then the node's interface on which the " +
 			"default gateway is configured will be used as the gateway " +
 			"interface. Only useful with \"init-gateways\"",
+		Destination: &cliConfig.Gateway.Interface,
 	},
 	cli.StringFlag{
 		Name: "gateway-nexthop",
@@ -431,26 +526,35 @@ var OVNGatewayFlags = []cli.Flag{
 			"of the node in question. If not specified, the default gateway" +
 			"configured in the node is used. Only useful with " +
 			"\"init-gateways\"",
-	},
-	cli.BoolFlag{
-		Name: "gateway-spare-interface",
-		Usage: "If true, assumes that \"gateway-interface\" provided can be " +
-			"exclusively used for the OVN gateway.  When true, only OVN" +
-			"related traffic can flow through this interface",
-	},
-	cli.BoolFlag{
-		Name: "gateway-local",
-		Usage: "If true, creates a local gateway (br-local) to let traffic reach " +
-			"host network and also exit host with iptables NAT",
+		Destination: &cliConfig.Gateway.NextHop,
 	},
 	cli.UintFlag{
 		Name: "gateway-vlanid",
 		Usage: "The VLAN on which the external network is available. " +
 			"Valid only for Shared or Spare Gateway interface mode.",
+		Destination: &cliConfig.Gateway.VLANID,
 	},
 	cli.BoolFlag{
-		Name:  "nodeport",
-		Usage: "Setup nodeport based ingress on gateways.",
+		Name:        "nodeport",
+		Usage:       "Setup nodeport based ingress on gateways.",
+		Destination: &cliConfig.Gateway.NodeportEnable,
+	},
+
+	// Deprecated CLI options
+	cli.BoolFlag{
+		Name:        "init-gateways",
+		Usage:       "DEPRECATED; use --gateway-mode instead",
+		Destination: &initGateways,
+	},
+	cli.BoolFlag{
+		Name:        "gateway-spare-interface",
+		Usage:       "DEPRECATED; use --gateway-mode instead",
+		Destination: &gatewaySpareInterface,
+	},
+	cli.BoolFlag{
+		Name:        "gateway-local",
+		Usage:       "DEPRECATED; use --gateway-mode instead",
+		Destination: &gatewayLocal,
 	},
 }
 
@@ -524,7 +628,17 @@ func setOVSExternalID(exec kexec.Interface, key, value string) error {
 	return nil
 }
 
-func buildKubernetesConfig(exec kexec.Interface, cli, file *config, defaults *Defaults) error {
+func buildKubernetesConfig(exec kexec.Interface, cli, file *config, saPath string, defaults *Defaults) error {
+	// token adn ca.crt may be from files mounted in container.
+	var saConfig KubernetesConfig
+	if data, err := ioutil.ReadFile(filepath.Join(saPath, kubeServiceAccountFileToken)); err == nil {
+		saConfig.Token = string(data)
+	}
+	if _, err2 := os.Stat(filepath.Join(saPath, kubeServiceAccountFileCACert)); err2 == nil {
+		saConfig.CACert = filepath.Join(saPath, kubeServiceAccountFileCACert)
+	}
+	overrideFields(&Kubernetes, &saConfig)
+
 	// Grab default values from OVS external IDs
 	if defaults.K8sAPIServer {
 		Kubernetes.APIServer = getOVSExternalID(exec, "k8s-api-server")
@@ -536,6 +650,18 @@ func buildKubernetesConfig(exec kexec.Interface, cli, file *config, defaults *De
 		Kubernetes.CACert = getOVSExternalID(exec, "k8s-ca-certificate")
 	}
 
+	// values for token, cacert, kubeconfig, api-server may be found in several places.
+	// Take the first found when looking in this order: command line options, config file,
+	// environment variables, service account files
+
+	envConfig := KubernetesConfig{
+		Kubeconfig: os.Getenv("KUBECONFIG"),
+		CACert:     os.Getenv("K8S_CACERT"),
+		APIServer:  os.Getenv("K8S_APISERVER"),
+		Token:      os.Getenv("K8S_TOKEN"),
+	}
+	overrideFields(&Kubernetes, &envConfig)
+
 	// Copy config file values over default values
 	overrideFields(&Kubernetes, &file.Kubernetes)
 	// And CLI overrides over config file and default values
@@ -555,6 +681,84 @@ func buildKubernetesConfig(exec kexec.Interface, cli, file *config, defaults *De
 		return fmt.Errorf("kubernetes API server URL scheme %q invalid", url.Scheme)
 	}
 
+	// Legacy service-cluster-ip-range CLI option overrides config file or --k8s-service-cidr
+	if serviceClusterIPRange != "" {
+		Kubernetes.ServiceCIDR = serviceClusterIPRange
+	}
+	if Kubernetes.ServiceCIDR == "" {
+		return fmt.Errorf("kubernetes service-cidr is required")
+	} else if _, _, err := net.ParseCIDR(Kubernetes.ServiceCIDR); err != nil {
+		return fmt.Errorf("kubernetes service network CIDR %q invalid: %v", Kubernetes.ServiceCIDR, err)
+	}
+
+	return nil
+}
+
+func buildGatewayConfig(ctx *cli.Context, cli, file *config) error {
+	// Copy config file values over default values
+	overrideFields(&Gateway, &file.Gateway)
+
+	cli.Gateway.Mode = GatewayMode(ctx.String("gateway-mode"))
+	if cli.Gateway.Mode == GatewayModeDisabled {
+		// Handle legacy CLI options
+		if ctx.Bool("init-gateways") {
+			cli.Gateway.Mode = GatewayModeShared
+			if ctx.Bool("gateway-spare-interface") {
+				cli.Gateway.Mode = GatewayModeSpare
+			} else if ctx.Bool("gateway-local") {
+				cli.Gateway.Mode = GatewayModeLocal
+			}
+		}
+	}
+	// And CLI overrides over config file and default values
+	overrideFields(&Gateway, &cli.Gateway)
+
+	if Gateway.Mode != GatewayModeDisabled {
+		validModes := []string{string(GatewayModeShared), string(GatewayModeSpare), string(GatewayModeLocal)}
+		var found bool
+		for _, mode := range validModes {
+			if string(Gateway.Mode) == mode {
+				found = true
+				break
+			}
+		}
+		if !found {
+			return fmt.Errorf("invalid gateway mode %q: expect one of %s", string(Gateway.Mode), strings.Join(validModes, ","))
+		}
+	}
+
+	// Options are only valid if Mode is not disabled
+	if Gateway.Mode == GatewayModeDisabled {
+		if Gateway.Interface != "" {
+			return fmt.Errorf("gateway interface option %q not allowed when gateway is disabled", Gateway.Interface)
+		}
+		if Gateway.NextHop != "" {
+			return fmt.Errorf("gateway next-hop option %q not allowed when gateway is disabled", Gateway.NextHop)
+		}
+		if Gateway.VLANID != 0 {
+			return fmt.Errorf("gateway VLAN ID option '%d' not allowed when gateway is disabled", Gateway.VLANID)
+		}
+	}
+	return nil
+}
+
+func buildDefaultConfig(cli, file *config) error {
+	overrideFields(&Default, &file.Default)
+	overrideFields(&Default, &cli.Default)
+
+	// Legacy cluster-subnet CLI option overrides config file or --cluster-subnets
+	if clusterSubnet != "" {
+		Default.RawClusterSubnets = clusterSubnet
+	}
+	if Default.RawClusterSubnets == "" {
+		return fmt.Errorf("cluster subnet is required")
+	}
+
+	var err error
+	Default.ClusterSubnets, err = ParseClusterSubnetEntries(Default.RawClusterSubnets)
+	if err != nil {
+		return fmt.Errorf("cluster subnet invalid: %v", err)
+	}
 	return nil
 }
 
@@ -580,23 +784,27 @@ func getConfigFilePath(ctx *cli.Context) (string, bool) {
 // constructs the global config object from them. It returns the config file
 // path (if explicitly specified) or an error
 func InitConfig(ctx *cli.Context, exec kexec.Interface, defaults *Defaults) (string, error) {
-	return InitConfigWithPath(ctx, exec, "", defaults)
+	return initConfigWithPath(ctx, exec, kubeServiceAccountPath, defaults)
 }
 
-// InitConfigWithPath reads the given config file (or if empty, reads the config file
+// InitConfigSa reads the config file and common command-line options and
+// constructs the global config object from them. It passes the service account directory.
+// It returns the config file path (if explicitly specified) or an error
+func InitConfigSa(ctx *cli.Context, exec kexec.Interface, saPath string, defaults *Defaults) (string, error) {
+	return initConfigWithPath(ctx, exec, saPath, defaults)
+}
+
+// initConfigWithPath reads the given config file (or if empty, reads the config file
 // specified by command-line arguments, or empty, the default config file) and
 // common command-line options and constructs the global config object from
 // them. It returns the config file path (if explicitly specified) or an error
-func InitConfigWithPath(ctx *cli.Context, exec kexec.Interface, configFile string, defaults *Defaults) (string, error) {
+func initConfigWithPath(ctx *cli.Context, exec kexec.Interface, saPath string, defaults *Defaults) (string, error) {
 	var cfg config
 	var retConfigFile string
+	var configFile string
 	var configFileIsDefault bool
 
-	// If no specific config file was given, try to find one from command-line
-	// arguments, or the platform-specific default config file path
-	if configFile == "" {
-		configFile, configFileIsDefault = getConfigFilePath(ctx)
-	}
+	configFile, configFileIsDefault = getConfigFilePath(ctx)
 
 	logrus.SetOutput(os.Stderr)
 
@@ -626,8 +834,6 @@ func InitConfigWithPath(ctx *cli.Context, exec kexec.Interface, configFile strin
 	}
 
 	// Build config that needs no special processing
-	overrideFields(&Default, &cfg.Default)
-	overrideFields(&Default, &cliConfig.Default)
 	overrideFields(&CNI, &cfg.CNI)
 	overrideFields(&CNI, &cliConfig.CNI)
 
@@ -651,7 +857,15 @@ func InitConfigWithPath(ctx *cli.Context, exec kexec.Interface, configFile strin
 		}
 	}
 
-	if err = buildKubernetesConfig(exec, &cliConfig, &cfg, defaults); err != nil {
+	if err = buildDefaultConfig(&cliConfig, &cfg); err != nil {
+		return "", err
+	}
+
+	if err = buildKubernetesConfig(exec, &cliConfig, &cfg, saPath, defaults); err != nil {
+		return "", err
+	}
+
+	if err = buildGatewayConfig(ctx, &cliConfig, &cfg); err != nil {
 		return "", err
 	}
 
@@ -879,26 +1093,37 @@ func (a *OvnAuthConfig) SetDBAuth() error {
 	return nil
 }
 
-func (a *OvnAuthConfig) updateIP(newIP string) error {
+func (a *OvnAuthConfig) updateIP(newIP []string, port string) error {
 	if a.Address != "" {
 		s := strings.Split(a.Address, ":")
 		if len(s) != 3 {
 			return fmt.Errorf("failed to parse OvnAuthConfig address %q", a.Address)
 		}
-		a.Address = s[0] + ":" + newIP + s[2]
+		var newPort string
+		if port != "" {
+			newPort = port
+		} else {
+			newPort = s[2]
+		}
+
+		newAddresses := make([]string, 0, len(newIP))
+		for _, ipAddress := range newIP {
+			newAddresses = append(newAddresses, s[0]+":"+ipAddress+":"+newPort)
+		}
+		a.Address = strings.Join(newAddresses, ",")
 	}
 	return nil
 }
 
-// UpdateOvnNodeAuth updates the host and URL in ClientAuth
+// UpdateOVNNodeAuth updates the host and URL in ClientAuth
 // for both OvnNorth and OvnSouth. It updates them with the new masterIP.
-func UpdateOvnNodeAuth(masterIP string) error {
+func UpdateOVNNodeAuth(masterIP []string, southboundDBPort, northboundDBPort string) error {
 	logrus.Debugf("Update OVN node auth with new master ip: %s", masterIP)
-	if err := OvnNorth.updateIP(masterIP); err != nil {
+	if err := OvnNorth.updateIP(masterIP, northboundDBPort); err != nil {
 		return fmt.Errorf("failed to update OvnNorth ClientAuth URL: %v", err)
 	}
 
-	if err := OvnSouth.updateIP(masterIP); err != nil {
+	if err := OvnSouth.updateIP(masterIP, southboundDBPort); err != nil {
 		return fmt.Errorf("failed to update OvnSouth ClientAuth URL: %v", err)
 	}
 	return nil
